/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2020 Nicolas Saenz Julienne
 * Author: Nicolas Saenz Julienne <nsaenzjulienne@suse.de>
 */

#ifndef _DT_BINDINGS_RASPBERRYPI_FIRMWARE_PWM_H
#define _DT_BINDINGS_RASPBERRYPI_FIRMWARE_PWM_H

#define RASPBERRYPI_FIRMWARE_PWM_POE		0
#define RASPBERRYPI_FIRMWARE_PWM_NUM		1

#endif
